<?php
/**
* 
*
* @author BTConsulting <BTConsulting.dev@gmail.com>
* @copyright BTConsulting
* @license http://www.opensource.org/licenses/osl-3.0.php Open-source licence 3.0
* @version 2.0
*
*/

class AdminSaleCategoryController extends ModuleAdminController {

    private $_nbProcessedCategories;
    private $_nb_processed_products;
    
    private $_categoryTree;
    private $_confmsg;
    
    public function __construct()
    {
		$this->bootstrap = true;
	
        $this->_nbProcessedCategories=0;
        $this->_nb_processed_products=0;
        
        $this->_categoryTree='';
        $this->_confmsg='';
    
        parent::__construct();
    }    
    
    public function setMedia()
    {
        parent::setMedia();
        
        $this->addJqueryUI(array(
            'ui.core',
            'ui.widget',
            'ui.slider',
            'ui.datepicker'
        ));
    
        $this->addJS(array(
            _PS_JS_DIR_.'jquery/plugins/timepicker/jquery-ui-timepicker-addon.js',
            _PS_JS_DIR_.'jquery/plugins/jquery.typewatch.js',
        ));

        $this->addCSS(array(
            _PS_JS_DIR_.'jquery/plugins/timepicker/jquery-ui-timepicker-addon.css',
        ));        
    }    
   
    public function display()
    {
        $categories = Category::getCategories($this->context->language->id,false);

        $root_category = Category::getRootCategory();
        $this->_recurseCategory($categories, $categories[$root_category->id_parent][$root_category->id], $root_category->id);
        $this->context->smarty->assign('categories', $this->_categoryTree);
		
		$shops = Shop::getShops();
		$countries = Country::getCountries($this->context->language->id);
		$groups = Group::getGroups($this->context->language->id);
		$currencies = Currency::getCurrencies();		
		
		$this->context->smarty->assign(array(
			'shops' => $shops,
			'currencies' => $currencies,
			'countries' => $countries,
			'groups' => $groups,
			'multi_shop' => Shop::isFeatureActive(),
			'link' => new Link()
		));		
        
        if ($this->_confmsg) {
            $this->context->smarty->assign('conf', $this->_confmsg);
		}
        
        parent::display();
    }  
    
    
    public function initProcess()
    {
        if (Tools::isSubmit('submitSetReductionSale')) {
            $this->action = 'apply_sale_category';
        } elseif (Tools::isSubmit('submitUnsetReductionSale')) {
            $this->action = 'remove_sale_category';
		}
    }
    
    public function processApplySaleCategory()
    {
        $this->_processSetCategoryRecurse(Tools::getValue('categories'),Tools::getValue('recurse_apply'));
        // die();
        if(empty($this->_errors)) {
            $this->_confmsg = count($this->_processed_categories).' '.$this->l('Categories and').' '.$this->_nb_processed_products.' '.$this->l('Products successfully updated.');
		}
    }
    
    public function processRemoveSaleCategory()
    {
        $this->_processUnsetCategoryRecurse(Tools::getValue('categories'),Tools::getValue('recurse_remove'));
        
        if(empty($this->_errors)) {
            $this->_confmsg = count($this->_processed_categories).' '.$this->l('Categories and').' '.$this->_nb_processed_products.' '.$this->l('Products successfully updated.');
		}
    }    

    private function _processSetCategoryRecurse($id_category, $recurse)
    {
        $sql = 'SELECT cp.`id_product`
				FROM `'._DB_PREFIX_.'category_product` cp
				WHERE cp.`id_category`='.$id_category;
		if (!empty($this->_processed_categories)) {
			$sql .= ' AND cp.`id_product` NOT IN (SELECT tmp.`id_product`
					FROM `'._DB_PREFIX_.'category_product` tmp
					WHERE tmp.`id_category` IN ('.implode(',',$this->_processed_categories).'))';
		}
        $query_result = Db::getInstance()->ExecuteS($sql);

        $this->_processed_categories[] = $id_category;
    
        foreach ($query_result as $row) {
            $this->_nb_processed_products += 1;
            
            $id_product = $row['id_product'];
      
            if(Tools::getValue('apply_reduction')) {
				$id_shop = Tools::getValue('sp_id_shop');
				$id_currency = Tools::getValue('sp_id_currency');
				$id_country = Tools::getValue('sp_id_country');
				$id_group = Tools::getValue('sp_id_group');
				$id_customer = Tools::getValue('sp_id_customer');
				// $price = 1;
				$from_quantity = Tools::getValue('sp_from_quantity');
				$reduction = (float)(Tools::getValue('sp_reduction'));
				$reduction_type = !$reduction ? 'amount' : Tools::getValue('sp_reduction_type');
				$from = Tools::getValue('sp_from');
				if (!$from)
					$from = '0000-00-00 00:00:00';
				$to = Tools::getValue('sp_to');
				if (!$to)
					$to = '0000-00-00 00:00:00';
											
				if ($reduction_type == 'percentage' && ((float)$reduction <= 0 || (float)$reduction > 100)) {
					$this->errors[] = Tools::displayError('Submitted reduction value (0-100) is out-of-range');
				} elseif ($this->_validateSpecificPrice($id_shop, $id_product, $id_currency, $id_country, $id_group, $id_customer, 1, $from_quantity, $reduction, $reduction_type, $from, $to, 0)) {
					$specificPrice = new SpecificPrice();
					$specificPrice->id_product = (int)$id_product;
					// $specificPrice->id_product_attribute = (int)$id_product_attribute;
					$specificPrice->id_shop = (int)$id_shop;
					$specificPrice->id_currency = (int)($id_currency);
					$specificPrice->id_country = (int)($id_country);
					$specificPrice->id_group = (int)($id_group);
					$specificPrice->id_customer = (int)$id_customer;
					$specificPrice->price = (float)(-1);
					// $specificPrice->price = (float)($price);
					$specificPrice->from_quantity = (int)($from_quantity);
					$specificPrice->reduction = (float)($reduction_type == 'percentage' ? $reduction / 100 : $reduction);
					$specificPrice->reduction_type = $reduction_type;
					$specificPrice->from = $from;
					$specificPrice->to = $to;
					if (!$specificPrice->add()) {
						$this->errors[] = Tools::displayError('An error occurred while updating the specific price.');
					}
				}					
					
            }
            if (Tools::getValue('apply_sale')) {
				$id_shop = Tools::getValue('sp_id_shop');
                $product = new Product($id_product, false, null, $id_shop);
                $product->on_sale = true;
                if (!$product->update()) {
                    $this->_errors[] = Tools::displayError('An error occurred while updating product '.$id_product.'.');
				}
            }
        }
        if ($recurse) {
            $sql='SELECT c.`id_category`
                  FROM `'._DB_PREFIX_.'category` c
                  WHERE c.`id_parent`='.$id_category.
                  ' AND c.`active`=1';
            $query_result = Db::getInstance()->ExecuteS($sql);
        
            foreach ($query_result as $row) {
                $this->_processSetCategoryRecurse($row['id_category'], $recurse);
            }
        }
    }
    
    private function _processUnsetCategoryRecurse($id_category, $recurse)
    {
        $sql='SELECT cp.`id_product`
              FROM `'._DB_PREFIX_.'category_product` cp
              WHERE cp.`id_category`='.$id_category;
		if (!empty($this->_processed_categories)) {
			$sql .= ' AND cp.`id_product` NOT IN (SELECT tmp.`id_product`
					FROM `'._DB_PREFIX_.'category_product` tmp
					WHERE tmp.`id_category` IN ('.implode(',',$this->_processed_categories).'))';
			echo 'sql:'.$sql;
		}			  
        $query_result = Db::getInstance()->ExecuteS($sql);
		
		$this->_processed_categories[] = $id_category;

        foreach ($query_result as $row) {
            $this->_nb_processed_products += 1;

            $id_product = $row['id_product'];

            if (Tools::getValue('remove_reduction')) {
                if(!SpecificPrice::deleteByProductId($id_product)) {
                    $this->_errors[] = Tools::displayError('An error occurred while deleting the specific price.');
				}
			}
            if (Tools::getValue('remove_sale')) {
                $product = new Product($id_product);
                $product->on_sale = false;
                if (!$product->update()) {
                    $this->_errors[] = Tools::displayError('An error occurred while updating product '.$id_product.'.');
				}
            }
        }
        
        if ($recurse) {
            $sql = 'SELECT c.`id_category`
					FROM `'._DB_PREFIX_.'category` c
					WHERE c.`id_parent`='.$id_category.
					' AND c.`active`=1';
            $query_result = Db::getInstance()->ExecuteS($sql);
        
            foreach($query_result as $row) {
                $this->_processUnsetCategoryRecurse($row['id_category'], $recurse);
            }
        }
    }

    private function _recurseCategory($categories, $current, $id_category = 1, $id_selected = 1)
    {
        $root_category = Category::getRootCategory();
        $this->_categoryTree .= '<option value="'.$id_category.'"'.(($id_selected == $id_category) ? ' selected="selected"' : '').'>'.str_repeat('&nbsp;', ($current['infos']['level_depth']-$root_category->level_depth) * 5).stripslashes($current['infos']['name']).'</option>';
        if (isset($categories[$id_category])) {
            foreach (array_keys($categories[$id_category]) as $key) {
                $this->_recurseCategory($categories, $categories[$id_category][$key], $key, $id_selected);
			}
		}
    }
	
	protected function _validateSpecificPrice($id_shop, $id_product, $id_currency, $id_country, $id_group, $id_customer, $price, $from_quantity, $reduction, $reduction_type, $from, $to, $id_combination = 0)
	{
		if (!Validate::isUnsignedId($id_shop) || !Validate::isUnsignedId($id_currency) || !Validate::isUnsignedId($id_country) || !Validate::isUnsignedId($id_group) || !Validate::isUnsignedId($id_customer))
			$this->errors[] = Tools::displayError('Wrong IDs');
		elseif ((!isset($price) && !isset($reduction)) || (isset($price) && !Validate::isNegativePrice($price)) || (isset($reduction) && !Validate::isPrice($reduction)))
			$this->errors[] = Tools::displayError('Invalid price/discount amount');
		elseif (!Validate::isUnsignedInt($from_quantity))
			$this->errors[] = Tools::displayError('Invalid quantity');
		elseif ($reduction && !Validate::isReductionType($reduction_type))
			$this->errors[] = Tools::displayError('Please select a discount type (amount or percentage).');
		elseif ($from && $to && (!Validate::isDateFormat($from) || !Validate::isDateFormat($to)))
			$this->errors[] = Tools::displayError('The from/to date is invalid.');
		elseif (SpecificPrice::exists($id_product, $id_combination, $id_shop, $id_group, $id_country, $id_currency, $id_customer, $from_quantity, $from, $to, false))
			$this->errors[] = Tools::displayError('A specific price already exists for these parameters.');
		else
			return true;
		return false;
	}	

}